# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: results_filters.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import filters_common_pb2 as filters__common__pb2
from . import result_status_pb2 as result__status__pb2
from . import results_fields_pb2 as results__fields__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15results_filters.proto\x12\x1b\x61rmonik.api.grpc.v1.results\x1a\x14\x66ilters_common.proto\x1a\x13result_status.proto\x1a\x14results_fields.proto\"\x8b\x01\n\x0c\x46ilterStatus\x12>\n\x05value\x18\x01 \x01(\x0e\x32/.armonik.api.grpc.v1.result_status.ResultStatus\x12;\n\x08operator\x18\x02 \x01(\x0e\x32).armonik.api.grpc.v1.FilterStatusOperator\"\xcb\x02\n\x0b\x46ilterField\x12\x37\n\x05\x66ield\x18\x01 \x01(\x0b\x32(.armonik.api.grpc.v1.results.ResultField\x12:\n\rfilter_string\x18\x02 \x01(\x0b\x32!.armonik.api.grpc.v1.FilterStringH\x00\x12\x36\n\x0b\x66ilter_date\x18\x03 \x01(\x0b\x32\x1f.armonik.api.grpc.v1.FilterDateH\x00\x12\x38\n\x0c\x66ilter_array\x18\x04 \x01(\x0b\x32 .armonik.api.grpc.v1.FilterArrayH\x00\x12\x42\n\rfilter_status\x18\x05 \x01(\x0b\x32).armonik.api.grpc.v1.results.FilterStatusH\x00\x42\x11\n\x0fvalue_condition\"C\n\nFiltersAnd\x12\x35\n\x03\x61nd\x18\x01 \x03(\x0b\x32(.armonik.api.grpc.v1.results.FilterField\">\n\x07\x46ilters\x12\x33\n\x02or\x18\x01 \x03(\x0b\x32\'.armonik.api.grpc.v1.results.FiltersAndB\x1e\xaa\x02\x1b\x41rmoniK.Api.gRPC.V1.Resultsb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'results_filters_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\252\002\033ArmoniK.Api.gRPC.V1.Results'
  _globals['_FILTERSTATUS']._serialized_start=120
  _globals['_FILTERSTATUS']._serialized_end=259
  _globals['_FILTERFIELD']._serialized_start=262
  _globals['_FILTERFIELD']._serialized_end=593
  _globals['_FILTERSAND']._serialized_start=595
  _globals['_FILTERSAND']._serialized_end=662
  _globals['_FILTERS']._serialized_start=664
  _globals['_FILTERS']._serialized_end=726
# @@protoc_insertion_point(module_scope)



from . import result_status_pb2 as _result_status_pb2
from . import task_status_pb2 as _task_status_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class EventSubscriptionRequest(_message.Message):
    __slots__ = ["session_id"]
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    session_id: str
    def __init__(self, session_id: _Optional[str] = ...) -> None: ...

class EventSubscriptionResponse(_message.Message):
    __slots__ = ["session_id", "task_status_update", "result_status_update", "result_owner_update", "new_task", "new_result"]
    class TaskStatusUpdate(_message.Message):
        __slots__ = ["task_id", "status"]
        TASK_ID_FIELD_NUMBER: _ClassVar[int]
        STATUS_FIELD_NUMBER: _ClassVar[int]
        task_id: str
        status: _task_status_pb2.TaskStatus
        def __init__(self, task_id: _Optional[str] = ..., status: _Optional[_Union[_task_status_pb2.TaskStatus, str]] = ...) -> None: ...
    class ResultStatusUpdate(_message.Message):
        __slots__ = ["result_id", "status"]
        RESULT_ID_FIELD_NUMBER: _ClassVar[int]
        STATUS_FIELD_NUMBER: _ClassVar[int]
        result_id: str
        status: _result_status_pb2.ResultStatus
        def __init__(self, result_id: _Optional[str] = ..., status: _Optional[_Union[_result_status_pb2.ResultStatus, str]] = ...) -> None: ...
    class ResultOwnerUpdate(_message.Message):
        __slots__ = ["result_id", "previous_owner_id", "current_owner_id"]
        RESULT_ID_FIELD_NUMBER: _ClassVar[int]
        PREVIOUS_OWNER_ID_FIELD_NUMBER: _ClassVar[int]
        CURRENT_OWNER_ID_FIELD_NUMBER: _ClassVar[int]
        result_id: str
        previous_owner_id: str
        current_owner_id: str
        def __init__(self, result_id: _Optional[str] = ..., previous_owner_id: _Optional[str] = ..., current_owner_id: _Optional[str] = ...) -> None: ...
    class NewTask(_message.Message):
        __slots__ = ["task_id", "payload_id", "origin_task_id", "status", "expected_output_keys", "data_dependencies", "retry_of_ids", "parent_task_ids"]
        TASK_ID_FIELD_NUMBER: _ClassVar[int]
        PAYLOAD_ID_FIELD_NUMBER: _ClassVar[int]
        ORIGIN_TASK_ID_FIELD_NUMBER: _ClassVar[int]
        STATUS_FIELD_NUMBER: _ClassVar[int]
        EXPECTED_OUTPUT_KEYS_FIELD_NUMBER: _ClassVar[int]
        DATA_DEPENDENCIES_FIELD_NUMBER: _ClassVar[int]
        RETRY_OF_IDS_FIELD_NUMBER: _ClassVar[int]
        PARENT_TASK_IDS_FIELD_NUMBER: _ClassVar[int]
        task_id: str
        payload_id: str
        origin_task_id: str
        status: _task_status_pb2.TaskStatus
        expected_output_keys: _containers.RepeatedScalarFieldContainer[str]
        data_dependencies: _containers.RepeatedScalarFieldContainer[str]
        retry_of_ids: _containers.RepeatedScalarFieldContainer[str]
        parent_task_ids: _containers.RepeatedScalarFieldContainer[str]
        def __init__(self, task_id: _Optional[str] = ..., payload_id: _Optional[str] = ..., origin_task_id: _Optional[str] = ..., status: _Optional[_Union[_task_status_pb2.TaskStatus, str]] = ..., expected_output_keys: _Optional[_Iterable[str]] = ..., data_dependencies: _Optional[_Iterable[str]] = ..., retry_of_ids: _Optional[_Iterable[str]] = ..., parent_task_ids: _Optional[_Iterable[str]] = ...) -> None: ...
    class NewResult(_message.Message):
        __slots__ = ["result_id", "owner_id", "status"]
        RESULT_ID_FIELD_NUMBER: _ClassVar[int]
        OWNER_ID_FIELD_NUMBER: _ClassVar[int]
        STATUS_FIELD_NUMBER: _ClassVar[int]
        result_id: str
        owner_id: str
        status: _result_status_pb2.ResultStatus
        def __init__(self, result_id: _Optional[str] = ..., owner_id: _Optional[str] = ..., status: _Optional[_Union[_result_status_pb2.ResultStatus, str]] = ...) -> None: ...
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    TASK_STATUS_UPDATE_FIELD_NUMBER: _ClassVar[int]
    RESULT_STATUS_UPDATE_FIELD_NUMBER: _ClassVar[int]
    RESULT_OWNER_UPDATE_FIELD_NUMBER: _ClassVar[int]
    NEW_TASK_FIELD_NUMBER: _ClassVar[int]
    NEW_RESULT_FIELD_NUMBER: _ClassVar[int]
    session_id: str
    task_status_update: EventSubscriptionResponse.TaskStatusUpdate
    result_status_update: EventSubscriptionResponse.ResultStatusUpdate
    result_owner_update: EventSubscriptionResponse.ResultOwnerUpdate
    new_task: EventSubscriptionResponse.NewTask
    new_result: EventSubscriptionResponse.NewResult
    def __init__(self, session_id: _Optional[str] = ..., task_status_update: _Optional[_Union[EventSubscriptionResponse.TaskStatusUpdate, _Mapping]] = ..., result_status_update: _Optional[_Union[EventSubscriptionResponse.ResultStatusUpdate, _Mapping]] = ..., result_owner_update: _Optional[_Union[EventSubscriptionResponse.ResultOwnerUpdate, _Mapping]] = ..., new_task: _Optional[_Union[EventSubscriptionResponse.NewTask, _Mapping]] = ..., new_result: _Optional[_Union[EventSubscriptionResponse.NewResult, _Mapping]] = ...) -> None: ...



from . import filters_common_pb2 as _filters_common_pb2
from . import result_status_pb2 as _result_status_pb2
from . import results_fields_pb2 as _results_fields_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class FilterStatus(_message.Message):
    __slots__ = ["value", "operator"]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    OPERATOR_FIELD_NUMBER: _ClassVar[int]
    value: _result_status_pb2.ResultStatus
    operator: _filters_common_pb2.FilterStatusOperator
    def __init__(self, value: _Optional[_Union[_result_status_pb2.ResultStatus, str]] = ..., operator: _Optional[_Union[_filters_common_pb2.FilterStatusOperator, str]] = ...) -> None: ...

class FilterField(_message.Message):
    __slots__ = ["field", "filter_string", "filter_date", "filter_array", "filter_status"]
    FIELD_FIELD_NUMBER: _ClassVar[int]
    FILTER_STRING_FIELD_NUMBER: _ClassVar[int]
    FILTER_DATE_FIELD_NUMBER: _ClassVar[int]
    FILTER_ARRAY_FIELD_NUMBER: _ClassVar[int]
    FILTER_STATUS_FIELD_NUMBER: _ClassVar[int]
    field: _results_fields_pb2.ResultField
    filter_string: _filters_common_pb2.FilterString
    filter_date: _filters_common_pb2.FilterDate
    filter_array: _filters_common_pb2.FilterArray
    filter_status: FilterStatus
    def __init__(self, field: _Optional[_Union[_results_fields_pb2.ResultField, _Mapping]] = ..., filter_string: _Optional[_Union[_filters_common_pb2.FilterString, _Mapping]] = ..., filter_date: _Optional[_Union[_filters_common_pb2.FilterDate, _Mapping]] = ..., filter_array: _Optional[_Union[_filters_common_pb2.FilterArray, _Mapping]] = ..., filter_status: _Optional[_Union[FilterStatus, _Mapping]] = ...) -> None: ...

class FiltersAnd(_message.Message):
    __slots__ = []
    AND_FIELD_NUMBER: _ClassVar[int]
    def __init__(self, **kwargs) -> None: ...

class Filters(_message.Message):
    __slots__ = []
    OR_FIELD_NUMBER: _ClassVar[int]
    def __init__(self, **kwargs) -> None: ...



import traceback
from concurrent import futures
from typing import Callable, Union

import grpc
from grpc import Channel

from .seqlogger import ClefLogger
from ..common import Output, HealthCheckStatus
from ..protogen.common.objects_pb2 import Empty
from ..protogen.common.worker_common_pb2 import ProcessReply, HealthCheckReply
from ..protogen.worker.agent_service_pb2_grpc import AgentStub
from ..protogen.worker.worker_service_pb2_grpc import WorkerServicer, add_WorkerServicer_to_server
from .taskhandler import TaskHandler


class ArmoniKWorker(WorkerServicer):
    def __init__(self, agent_channel: Channel, processing_function: Callable[[TaskHandler], Output], health_check: Callable[[], HealthCheckStatus] = lambda: HealthCheckStatus.SERVING, logger=ClefLogger.getLogger("ArmoniKWorker")):
        """Creates a worker for ArmoniK

        Args:
            agent_channel: gRPC channel for the agent
            processing_function: Function that will be called when a
                task needs processing
            health_check: Function that will be called to check the
                health of the worker. Defaults to a simple "Serving"
                reply
            logger: Logger used for the worker, defaults to a logger
                ArmoniKWorker compatible with Seq
        """
        self.health_check = health_check
        self.processing_function = processing_function
        self._client = AgentStub(agent_channel)
        self._logger = logger

    def start(self, endpoint: str):
        """Starts the worker

        Args:
            endpoint: endpoint from which to listen to requests
        """
        server = grpc.server(futures.ThreadPoolExecutor(max_workers=1))
        add_WorkerServicer_to_server(self, server)
        server.add_insecure_port(endpoint)
        server.start()
        server.wait_for_termination()

    def Process(self, request_iterator, context) -> Union[ProcessReply, None]:
        try:
            self._logger.debug("Received task")
            task_handler = TaskHandler.create(request_iterator, self._client)
            return ProcessReply(output=self.processing_function(task_handler).to_message())
        except Exception as e:
            self._logger.exception(f"Failed task {''.join(traceback.format_exception(type(e) ,e, e.__traceback__))}", exc_info=e)

    def HealthCheck(self, request: Empty, context) -> HealthCheckReply:
        return HealthCheckReply(status=self.health_check().value)
